import {c} from "../ixpcountry/app.js";
import {close_details} from "../ixpcountry/code.js";
import IpmapCrowdsourcingComp from "../ixpcountry/IpmapCrowdsourcingComp.js";

// .format() for strings
// First, checks if it isn't implemented yet.
if (!String.prototype.format) {
  String.prototype.format = function() {
    var args = arguments;
    return this.replace(/{(\d+)}/g, function(match, number) { 
      return typeof args[number] != 'undefined'
        ? args[number]
        : match
      ;
    });
  };
}

function defaultDict(defaultValueFn) {
  const handler = {
    get: (obj, prop) => {
      if(!(prop in obj)) obj[prop] = defaultValueFn();
      return obj[prop];
    },
  };
  return new Proxy({}, handler);
}

let nestedDefaultDict = () => defaultDict(nestedDefaultDict);
let results = nestedDefaultDict();

// AF 2022-09-05: these definitions were previously defined declared with no `let` or `var`, assuming
// the objects would be declared in a global context (?)
let arrayOfLines, RTTs_str, tmp_l, min, hostName_str, ASN_str, ASN_array, ASN, Location_str, json_file, u_proto,
    as_path_str, tmp_as_path, maxMinRTT;

// requires files in the right place
// requires jquery to have been loaded
// args:
//   d: d3 data elt
//   jquery_elt: a jquery_elt to put the resulting html at
export function jedi_cell_detail_to_jquery_elt( proto, src_id , dst_id , jquery_elt ) {
   json_file = "../common/details/{0}/{1}/{2}/latest.json".format( proto, src_id, dst_id );
   $.ajax({url: json_file,
      async: true
   }).done(function( data ) {
      let txt = '<pre>{0}</pre>'.format( data['tracetxt'] );
      jquery_elt.html( txt );
   });
}


// Display the source and destination ASN of each cell
export function jedi_cell_show_source_dest_asn( proto, src_as , dst_as , source_asn, dest_asn, pageX, pageY) {


        source_asn.style("display", "block");  //The tooltip appears
        dest_asn.style("display", "block");  //The tooltip appears

        source_asn.html( "Source ASN: <b>" + src_as + '</b></br><small>' + hoverGetData(src_as) + "</small> "  )
        .style("left", pageX - 270 + "px")
        .style("top", pageY + 35 + "px");

        dest_asn.html( "Dest ASN: <b>" + dst_as + '</b></br><small>' + hoverGetData(dst_as) + "</small> "  )
        .style("left", pageX - 110 + "px")
        .style("top", pageY - 70 + "px");


}


let cachedData = Array();

export function hoverGetData(arg_ASN){

    ASN_array = (arg_ASN.match(/\d+\.\d+|\d+\b|\d+(?=\w)/g) || [] ).map(function (v) {return +v;}); //=> []
    ASN = ASN_array[0]

    if(!Number.isInteger(Number(ASN))){
        return
    }

    if(ASN in cachedData){
        return cachedData[ASN];
    }

    let localData = "error";

    $.ajax('https://stat.ripe.net/data/as-names/data.json?resource=AS' + Number(ASN), {
        async: false,
        success: function(data){
            localData = data.data.names[Number(ASN)];
        }
    });

    cachedData[ASN] = localData;
    return localData;
}


// Display the traceroute information
export function jedi_cell_show_traceroute_on_click( proto, src_id , dst_id , traceroute_detail, pageX, pageY) {

  if (proto === "v4"){
    u_proto = "v6";
  }else{
    u_proto = "v4";
  }

  json_file = "../common/details/{0}/{1}/{2}/latest.json".format( proto, src_id, dst_id );
    $.ajax({url: json_file,
      async: true


    ,"success": function(data) {
        results[proto][src_id][dst_id] = data;

      as_path_str = "AS("+ proto + ") Path:</br>"
      for(let i in data['as_links']['ordered_as_path']){
        as_path_str += "{0}) ".format((parseInt(i) + 1)) + data['as_links']['ordered_as_path'][i] + "</br>";
      }
      let parsed_traceroute_one = parse_traceroute_info(data);

      let txt = '<button style="text-align:center" type="button" onclick="close_details(0)">Close</button>';
      txt += '<pre><div id="as_path_fw" style="width: 100%;height: 164px;"><div id="as_path_a" style="width: 50%;float:left;"><div style="font-size:18px;margin-top:6px;margin-bottom:5px;">IP' + proto + ' Traceroute: </div>'
      txt +='<div class="as_path">' + as_path_str + '</div></div></div></br>'
      txt += '<div style="width:100%;"><h3>{0}</h3><b>{1}</b><div id="bars_one"></div><div id="trac_2"></div></div>'.format( proto, parsed_traceroute_one['first_line'] );
      txt += '<small>Grey bar: max min RTT of all hops.</small></br><small>Light-blue bar: min RTT of the hop.</small></pre>'
      traceroute_detail.style("display", "block");


      traceroute_detail.html( txt )
        .style("left", (pageX + 20) + "px")
        .style("top", pageY + "px")


      json_file = "../common/details/{0}/{1}/{2}/latest.json".format( u_proto, src_id, dst_id );



      $.ajax({url: json_file,
          async: true


          ,"success": function(data_u) {

                results[u_proto][src_id][dst_id] = data_u;

              tmp_as_path = "";
              as_path_str = "AS("+ u_proto + ") Path:</br>"

              let parsed_traceroute_two = parse_traceroute_info(data_u);

              for(let i in data_u['as_links']['ordered_as_path']){
                as_path_str += "{0}) ".format((parseInt(i) + 1)) + data_u['as_links']['ordered_as_path'][i] + "</br>";
              }
              tmp_as_path += '<div id="as_path_b" style="width:50%;float:right;"><div style="font-size:18px;margin-top:6px;margin-bottom:5px;">IP' + u_proto + ' Traceroute: </div>'
              tmp_as_path +='<div class="as_path">' + as_path_str + '</div></br>'

              $("#as_path_fw").append(tmp_as_path);
              $("#trac_2").append('<h3>{0}</h3><b>{1}</b><div id="bars_two"></div>'.format( u_proto,  parsed_traceroute_two['first_line']));

              maxMinRTT = parsed_traceroute_one['maxMinRTT']

              if(maxMinRTT < parsed_traceroute_two['maxMinRTT']){
                maxMinRTT = parsed_traceroute_two['maxMinRTT']
              }

              plot_bars(parsed_traceroute_one, maxMinRTT ,"#bars_one", proto, src_id, dst_id)

              plot_bars(parsed_traceroute_two, maxMinRTT ,"#bars_two", u_proto, src_id, dst_id)

          },


          "error": function(jqXHR, status, error) {

            plot_bars(parsed_traceroute_one, parsed_traceroute_one['maxMinRTT'] ,"#bars_one", proto, src_id, dst_id)

          }

    });

  },

    "error": function(jqXHR, status, error) {

  }


  });
}


export function parse_traceroute_info(data){

    let traceroute_txt = data['tracetxt']
    let result = results[`v${data.protocol}`][data.src_prb_id][data.dst_prb_id]

  arrayOfLines = traceroute_txt.match(/[^\r\n]+/g);

  let match_AS = /\(.*\)/
  let match_hostName = /\s[a-z0-9-.:]+ /;
  let match_RTTs = /\[([0-9]+[.][0-9]*)[(,\s+)]*(([0-9]+[.])[0-9]*[(,\s+)]*)?(([0-9]+[.])[0-9]*[(,\s+)]*)?\]/;
  let match_location = /\|.*\|/

  let traceroute_obj = {ASN: new Array(), hostName: new Array(), rtts: new Array(), min: new Array(), maxMinRTT: null, location: new Array(), first_line: "", hopNumbers: new Array(), ips: new Array()}

  traceroute_obj['first_line'] = arrayOfLines[0]
  //Parse every line of the traceroute
    let previousHop;
  for(let i = 1; i < arrayOfLines.length; i++){
    //RTTs
      let line = arrayOfLines[i];
      let hopNumber = line.split(' ')[0];

      let atlasHopResult = result['result'].filter(res => res.hop===Number(hopNumber))[0].result;
      // console.log(line, line.match(/\*/g), atlasHopResult);
      traceroute_obj['hopNumbers'].push(hopNumber);

      RTTs_str = line.match(match_RTTs);

    if(RTTs_str == null){
        let stars = line.match(/\*/g);
        traceroute_obj['rtts'].push(stars);
        traceroute_obj['min'].push(0);

    }else{
      RTTs_str = RTTs_str[0].replace('[','').replace(']','');
      tmp_l = RTTs_str.split(",").map(Number);
      traceroute_obj['rtts'].push(tmp_l);

      min = get_min(tmp_l)
      traceroute_obj['min'].push(min)


      if(traceroute_obj['maxMinRTT'] == null || traceroute_obj['maxMinRTT'] < min){
        traceroute_obj['maxMinRTT'] = min;
      }
    }

    //HostName
    hostName_str = line.match(match_hostName)
      let ips = Array.from(new Set(atlasHopResult.filter(r => r.from).map(r => r.from))).join(', ');
    if(hostName_str == null){
      traceroute_obj['hostName'].push('Unknown');
      traceroute_obj['ips'].push(ips);

    }else{
        let hostName = line.match(match_hostName)[0].replaceAll(' ', '');
        // console.log(`${hostName}`, `${ips}`, String(hostName).valueOf() !== String(ips).valueOf())
        traceroute_obj['hostName'].push(hostName);
        traceroute_obj['ips'].push(ips);
    }

    //AS
    ASN_str = line.match(match_AS)
    if(ASN_str == null){
      traceroute_obj['ASN'].push('(AS*)');
    }else{
      traceroute_obj['ASN'].push(line.match(match_AS)[0]);
    }

    //Location
    Location_str = line.match(match_location)
    if(Location_str == null){
      traceroute_obj['location'].push('|*|')
    }else{
       traceroute_obj['location'].push(line.match(match_location)[0])
    }

    previousHop = hopNumber;
  }

  return traceroute_obj;

}

export function plot_bars(traceroute_obj, maxMinRTT, where, proto, src, dst){
    /*
    `proto`, `src`, and `dst` are used to re-fetch info about the *actual* IPs in the
    traceroute (and not the hostNames present in `traceroute_obj`)
     */

    let msmID = traceroute_obj.first_line.split(' ')[1].split(':')[1];
    let prbId = traceroute_obj.first_line.split(' ')[2].split(':')[1];
  let code_to_append = `<div class="bar-chart"><a style="font-size: .75em" href="https://atlas.ripe.net/api/v2/measurements/${msmID}/results/?probe_ids=${prbId}">RIPE Atlas measurement result</a><div class="chart clearfix">`

    let hopNumberPrevious = '';
    let packetNumber = 1;
    let vueComponents = [];

  for (let i = 0; i < traceroute_obj['ASN'].length; i++){
    let text_str = ""
    let percent = 0
    if(traceroute_obj['min'][i] !== '*'){
      percent = (traceroute_obj['min'][i]/maxMinRTT) * 100
    }

    let maxHopNumberStringLength = Math.max(...traceroute_obj['hopNumbers'].map(h => h.length))
    let maxAsnStringLength = Math.max(...traceroute_obj['ASN'].map(h => h.length))


    let rtts_str = traceroute_obj['rtts'][i].map(rtt => rtt === '*' ? rtt : `${rtt} ms`).join(' ')

    // we've created the Vue component `c` and we should be able to use it now
      let asn = traceroute_obj['ASN'][i].padEnd(maxAsnStringLength);
      let nullAsn = asn.valueOf() === '()'.valueOf();

      let atlasResults = results[proto][src][dst]['result'];
      let hopNumber = Number(traceroute_obj['hopNumbers'][i]);
      let atlasResultHop = atlasResults.filter(atlasResult => atlasResult.hop === hopNumber)[0];
      let ipsFromAtlasResult = atlasResultHop['result'].map(r => r['from']);



      // let ipsSetFromAtlasResult = atlasResultHop ? new Set(ipsFromAtlasResult) : new Set();
      // ipsSetFromAtlasResult.delete(undefined);

      // let location = traceroute_obj['location'][i];

      let hopNumberString;
      if(hopNumber === hopNumberPrevious) {
          hopNumberString = ' ';
          packetNumber++;
      } else {
          hopNumberString = `${hopNumber}`
          hopNumberPrevious = hopNumber;
          packetNumber = 1;
      }
      hopNumberString = hopNumberString.padEnd(maxHopNumberStringLength);
      let ipFromAtlasResult = ipsFromAtlasResult[packetNumber-1]
      let hostName = traceroute_obj['hostName'][i].replaceAll(' ', '');
      hostName = hostName === 'Unknown'? '' : hostName;
      let hostNameString = ipFromAtlasResult === undefined || hostName === ipFromAtlasResult ? hostName : `${hostName} (${ipFromAtlasResult})`;

      // console.log(
      //     hopNumber,
      //     ipsFromAtlasResult,
      //     traceroute_obj['hostName'],
      //     ipsFromAtlasResult[packetNumber-1]
      // );

      // let editLink = ipFromAtlasResult !== undefined ? ` <a href="#" onclick='c.addContribution("${ipFromAtlasResult}")'>geo: (${ipFromAtlasResult})</a>` : '';
      // let editLink = ipFromAtlasResult !== undefined ? ` <input
      //     placeholder="Geolocate ${ipFromAtlasResult}..."
      //     style="border-width: 0px; border-bottom-width: 1px; width: ${ipFromAtlasResult.length + "Geolocate ...".length}em"
      // />` : '';
      let elementId = `id_${hopNumber}_${packetNumber}`;
      vueComponents.push(
          {
              component: Vue.createApp(
                  IpmapCrowdsourcingComp,
                  {
                      ip: ipFromAtlasResult,
                      cities: c.getCities()
                  }
              ),
              el: elementId
          }
      );
      let editLink = `<span id="${elementId}"></span>`



      hopNumberPrevious = hopNumber;


      text_str = `${hopNumberString} ${asn} ${hostNameString} ${rtts_str} ${editLink}`; // ${location}

    code_to_append += '<div class="item"><div class="bar"><div class="item-progress" data-percent="' + percent + '">'
    code_to_append += '<div style="margin-left:4px;width:820px;overflow-y:hidden;overflow-x:scroll;"><span class="title">' + text_str + '</span></div></div></div></div>'


  }
  code_to_append += '</div></div>'

  $(where).append(code_to_append);

  for( let {component, el} of vueComponents)
      component.mount(`#${el}`);

  barChart();

}


export function get_min(values){
  return Math.min.apply(null,values)
}

export function get_median(values){
  values.sort((a, b) => a - b);
  return (values[(values.length - 1) >> 1] + values[values.length >> 1]) / 2
}

export function barChart(){
    $('.bar-chart').find('.item-progress').each(function(){
        let itemProgress = $(this),
        itemProgressWidth = $(this).parent().width() * ($(this).data('percent') / 100);
        itemProgress.css('width', itemProgressWidth);
    });
};