let id = 1;

export default {
    data() {
        return {
            id: null,
            searchQuery: '',
            searchResult: '',
            hits: 0,
            total: 0,
            citiesFiltered: [],
            placeholder: `type a city for ${this.ip}`,
            submitted: false,
            ipmapInfo: undefined
        }
    },
    created() {
        this.total = this.cities.length;
    },
    async mounted() {
        this.id = id++;
        await this.fetchIpmapInfo();
    },
    props: {
        ip: String,
        cities: Array
    },
    methods: {
        async fetchIpmapInfo() {

            if (this.ip === undefined)
                return undefined

            if(this.ipmapInfo)
                return this.ipmapInfo

            this.ipmapInfo = await fetch(
                `https://ipmap.ripe.net/api/v1/locate/${this.ip}`
            ).then(
                r => r.json()
            );


            if(this.ipmapInfo.locations) {
                let bestGuess = this.ipmapInfo.locations.sort((a,b) => (a.score > b.score) ? 1 : -1)[0];
                if(bestGuess) {

                    let bestGuessCity = {
                        city_name_ascii: bestGuess.cityName || bestGuess.cityNameAscii,
                        state_name: bestGuess.stateName,
                        country_code_alpha2: bestGuess.countryCodeAlpha2
                    };
                    console.log(bestGuess, bestGuess.countryCodeAlpha2, this.label(bestGuessCity))
                    let contributions = Object.keys(bestGuess.contributions);
                    let placeholderStart = contributions.includes('crowdsourced') ? 'Crowdsourced' : 'Our best guess'
                    this.placeholder = `${placeholderStart}: ${this.label(bestGuessCity)}`;
                }
            }



        },
        searchCity() {

            let searchString = this.searchQuery.toLowerCase();
            let filter = this.cities.filter(
                city => city.city_name_ascii.toLowerCase().includes(searchString) ||
                    city.state_name.toLowerCase().includes(searchString) ||
                    city.country_name.toLowerCase().includes(searchString) ||
                    city.country_code_alpha2.toLowerCase().includes(searchString) ||
                    city.country_code_alpha3.toLowerCase().includes(searchString) ||
                    city.state_iso_code.toLowerCase().includes(searchString) ||

                    this.label(city).toLowerCase().includes(searchString)
            );
            this.citiesFiltered = filter.sort(
                (c1, c2) => this.label(c1).localeCompare(this.label(c2))
            );
            this.hits = filter.length;

            return filter;
        },
        label(city) {
            return `${city.city_name_ascii}, ${city.state_name} (${city.country_code_alpha2})`
        },
        log: console.log,
        submit() {
            this.submitted = false;
            console.log('submitting...');
            new Promise(
                resolve => setTimeout(resolve, 1000)
            ).then(
                d => {
                    console.log('submitted', this)
                    this.$emit('addContribution(this.ip)')
                    this.submitted = true;
                }
            );

        },
        getStyle() {
            return {
                'border-width': '0px',
                'height': '1em',
                'background-color': this.submitted ? 'green' : '',
                'opacity': .75,
                'width': `${10*this.placeholder.length}px`,
            }
        },
        // getPlaceholder() {
        //     if (this.cities.length === 0)
        //         return 'loading cities...'
        //
        //     if(this.ipmapInfo && this.ipmapInfo.locations) {
        //
        //         console.log('>>>', this.ipmapInfo);
        //         console.log('>>>', this.ipmapInfo.locations);
        //         console.log('>>>', this.ipmapInfo.locations.sort((a,b) => (a.score > b.score) ? 1 : -1));
        //         console.log('>>>', this.ipmapInfo.locations.sort((a,b) => (a.score > b.score) ? 1 : -1)[0]);
        //         console.log('>>>', this.label(this.ipmapInfo.locations.sort((a,b) => (a.score > b.score) ? 1 : -1)[0]));
        //
        //         let bestGuess = this.ipmapInfo.locations.sort((a,b) => (a.score > b.score) ? 1 : -1)[0];
        //         let bestGuessCity = {
        //             city_name_ascii: bestGuess.cityNameAscii,
        //             state_name: bestGuess.stateName,
        //             country_code_alpha_2: bestGuess.countryCodeAlpha2
        //         };
        //         return `our best guess: ${this.label(bestGuessCity)}`
        //     }
        //     return `type a city for ${this.ip}`;
        // }
    },
    template: `
  <input 
      v-if="ip"
      v-model="searchQuery"
      @keyup="searchCity()"
      @keyup.enter="submit()"
      @keydown.tab="submit()"
      :list="id"
      :tabindex="id"
      :placeholder="placeholder"
      :disabled="cities.length==0"
      :style="getStyle()"
  />
  <datalist :id="id">
      <option v-for="city in citiesFiltered">{{ label(city) }}</option>
  </datalist>
  `
}