const {createApp} = Vue
import {csv} from "https://cdn.skypack.dev/d3-fetch@3";
import IpmapCrowdsourcingComp from './IpmapCrowdsourcingComp.js'


let id = 0;

let app = createApp({
    data() {
        return {
            title: `RIPE IPmap crowdsourcing widget`,
            appPlaceholder: `The list is empty. Add contributions by clicking on a cell in the matrix and clicking on the "edit geolocation" link...`,
            message: `Let others know the geolocation of Internet router's IP addresses `,
            contributions: [],
            cities: []
        }
    },
    created()
    {
        this.fetchCities();
    },
    components: {
      IpmapCrowdsourcingComp,
    },
    methods: {
        addContribution(ip) {

            // unique addresses only
            if (this.contributions.filter(e => e.ip === ip).length)
                return

            this.contributions.push(
                {
                    ip: ip,
                    id: id++
                }
            );
        },
        removeContribution(contribution) {
            this.contributions = this.contributions.filter(con => con.id !== contribution.id);
        },
        fetchCities(){
            csv("cities.csv").then((data) => {
                this.cities = data;
            });
        },
        getCities() {
            console.log(`getting ${this.cities.length} cities...`)
            return this.cities
        }
    }
});
app.provide('addContribution');
export let c = app.mount('#ipmap-panel');
